/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI;

import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.*;
import org.hibernate.SessionFactory;
import org.hibernate.boot.registry.StandardServiceRegistryBuilder;
import org.hibernate.cfg.Configuration;
import org.hibernate.service.ServiceRegistry;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class HibernateConnectionUtility {

    private static SessionFactory sessionFactory;
    private static Properties properties;

    public static SessionFactory getSessionFactory(){
        if (sessionFactory == null){

            try (InputStream input = HibernateConnectionUtility.class.getClassLoader().getResourceAsStream("config.properties")) {

                properties = new Properties();

                if (input == null) {
                    System.out.println("Sorry, unable to find configuration file.");
                    return null;
                }
                properties.load(input);

            } catch (IOException ex) {
                ex.printStackTrace();
            }

            Configuration configuration = new Configuration();
            configuration.addAnnotatedClass(Module.class);
            configuration.addAnnotatedClass(ModulePort.class);
            configuration.addAnnotatedClass(ModuleParameter.class);
            configuration.addAnnotatedClass(Operation.class);
            configuration.addAnnotatedClass(Node.class);
            configuration.addAnnotatedClass(Edge.class);
            configuration.addAnnotatedClass(NodeParameter.class);
            configuration.setProperty("hibernate.dialect", properties.getProperty("hibernate.dialect"));
            configuration.setProperty("hibernate.connection.driver_class", properties.getProperty("hibernate.connection.driver_class"));
            configuration.setProperty("hibernate.connection.url", properties.getProperty("hibernate.connection.url"));
            configuration.setProperty("hibernate.connection.username", properties.getProperty("hibernate.connection.username"));
            configuration.setProperty("hibernate.connection.password", properties.getProperty("hibernate.connection.password"));
            configuration.setProperty("hibernate.show_sql", "true");
            configuration.setProperty("hibernate.hbm2ddl.auto", "update");

            ServiceRegistry serviceRegistry = new StandardServiceRegistryBuilder()
                    .applySettings(configuration.getProperties()).build();
            sessionFactory = configuration.buildSessionFactory(serviceRegistry);
        }
        return sessionFactory;
    }
}
