/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI;

import javafx.animation.PauseTransition;
import javafx.animation.ScaleTransition;
import javafx.animation.SequentialTransition;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.layout.Pane;
import javafx.scene.layout.StackPane;
import javafx.util.Duration;

import java.io.IOException;

public class InformationDialog {

    @FXML
    private Label primaryLabel;
    @FXML
    private Label secondaryLabel;

    private Pane pane;

    private ScaleTransition openingScale = new ScaleTransition();
    private ScaleTransition closingScale = new ScaleTransition();

    private PauseTransition hide = new PauseTransition(Duration.seconds(10));

    private SequentialTransition animation = new SequentialTransition(openingScale, closingScale);

    public InformationDialog() throws IOException {
        FXMLLoader loader = new FXMLLoader(App.class.getClassLoader().getResource("view/window2.fxml"));
        loader.setController(this);
        pane = loader.load();

        StackPane.setAlignment(pane, Pos.TOP_CENTER);
        StackPane.setMargin(pane, new Insets(10,10,10,10));

        openingScale.setFromX(0.01);
        openingScale.setFromY(0.01);
        openingScale.setToY(1.0);
        openingScale.setDuration(Duration.seconds(0.2));
        openingScale.setNode(pane);

        closingScale.setFromX(0.01);
        closingScale.setToX(1.0);
        closingScale.setDuration(Duration.seconds(0.2));
        closingScale.setNode(pane);

        hide.setOnFinished(e -> {
            ((Pane) App.getScene().getRoot()).getChildren().remove(pane);
        });
    }

    public void openDialog() {
        ((Pane) App.getScene().getRoot()).getChildren().add(pane);
        animation.play();
        hide.playFromStart();
    }
    public void close(){
        ((Pane) App.getScene().getRoot()).getChildren().remove(pane);
    }

    public void loadData(String primaryLabel, String secondaryLabel){
        this.primaryLabel.setText(primaryLabel);
        this.secondaryLabel.setText(secondaryLabel);
    }
}
