/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI;

import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.PopUpWindows.*;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Node;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.NodeParameter;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class PopUpWindow {

    public static boolean getModuleDeleteResult() throws IOException {
        return showChoiceWindow("Do you want to delete selected Module?",
                "Module and all the associated ports & parameters will be deleted.");
    }

    public static boolean getOperationDeleteResult() throws IOException {
        return showChoiceWindow("Do you want to delete selected Operation?",
                "Operation and all its content will be deleted.");
    }

    public static void showDuplicateNameWindow() throws IOException {
        InformationDialog informationDialog = new InformationDialog();
        informationDialog.loadData("Name already taken", "Please, try a different one.");
        informationDialog.openDialog();
    }

    public static void showRequiredNameWindow() throws IOException {
        InformationDialog informationDialog = new InformationDialog();
        informationDialog.loadData("Name missing", "Name of the module is a required parameter.");
        informationDialog.openDialog();
    }

    public static void showInvalidPortNumberWindow() throws IOException {
        InformationDialog informationDialog = new InformationDialog();
        informationDialog.loadData("Invalid port number", "Use a valid port number [0 - 65535].");
        informationDialog.openDialog();
    }

    public static void showDuplicatePortNumberWindow() throws IOException {
        InformationDialog informationDialog = new InformationDialog();
        informationDialog.loadData("Port number taken", "he port is already being used for selected module.");
        informationDialog.openDialog();
    }

    public static void showCycleDetected() throws IOException {
        InformationDialog informationDialog = new InformationDialog();
        informationDialog.loadData("Cycle detected", "Circular dependencies are not allowed.");
        informationDialog.openDialog();
    }

    public static void showAdjacentVertices() throws IOException {
        InformationDialog informationDialog = new InformationDialog();
        informationDialog.loadData("Unable to create dependency", "The dependency between nodes already exists.");
        informationDialog.openDialog();
    }

    public static List<String> showRepositories() throws IOException {
        Scene previousPane = App.getScene();

        Rectangle dimmedBackground = new Rectangle(App.getScene().getWidth(), App.getScene().getHeight());
        dimmedBackground.setFill(Color.rgb(116,118,120,0.6));

        ((Pane)previousPane.getRoot()).getChildren().add(dimmedBackground);

        Stage selectRespositoryStage = new Stage();
        selectRespositoryStage.initModality(Modality.APPLICATION_MODAL);
        selectRespositoryStage.initStyle(StageStyle.TRANSPARENT);

        SelectRepositoryWindowController selectRepositoryWindowController = new SelectRepositoryWindowController(selectRespositoryStage);
        FXMLLoader loader = new FXMLLoader(App.class.getClassLoader().getResource("view/selectRepositoryWindow.fxml"));
        loader.setController(selectRepositoryWindowController);

        Scene scene = new Scene(loader.load());
        scene.setFill(Color.TRANSPARENT);
        selectRespositoryStage.setScene(scene);
        selectRespositoryStage.showAndWait();

        ((Pane)previousPane.getRoot()).getChildren().remove(dimmedBackground);

        List<String> list = new ArrayList<>();
        list.add(selectRepositoryWindowController.getRepositoryName());
        list.add(selectRepositoryWindowController.getTag());
        return list;
    }

    public static Optional<String> getOperationName() throws IOException {
        Scene previousPane = App.getScene();

        Rectangle dimmedBackground = new Rectangle(App.getScene().getWidth(), App.getScene().getHeight());
        dimmedBackground.setFill(Color.rgb(116,118,120,0.6));

        ((Pane)previousPane.getRoot()).getChildren().add(dimmedBackground);

        Stage getOperationName = new Stage();
        getOperationName.initModality(Modality.APPLICATION_MODAL);
        getOperationName.initStyle(StageStyle.TRANSPARENT);

        TextFieldWindowControler textFieldWindowController = new TextFieldWindowControler(getOperationName);
        FXMLLoader loader = new FXMLLoader(App.class.getClassLoader().getResource("view/textFieldWindow.fxml"));
        loader.setController(textFieldWindowController);

        Scene scene = new Scene(loader.load());
        scene.setFill(Color.TRANSPARENT);
        getOperationName.setScene(scene);
        getOperationName.showAndWait();

        ((Pane)previousPane.getRoot()).getChildren().remove(dimmedBackground);

        if (textFieldWindowController.getSelected()){
            return Optional.of(textFieldWindowController.getText());
        }
        return Optional.empty();
    }

    public static Optional<NodeParameter> addNodeParameter(Node node) throws IOException {
        Scene previousPane = App.getScene();

        Rectangle dimmedBackground = new Rectangle(App.getScene().getWidth(), App.getScene().getHeight());
        dimmedBackground.setFill(Color.rgb(116,118,120,0.6));

        ((Pane)previousPane.getRoot()).getChildren().add(dimmedBackground);

        Stage addNodeParameterStage = new Stage();
        addNodeParameterStage.initModality(Modality.APPLICATION_MODAL);
        addNodeParameterStage.initStyle(StageStyle.TRANSPARENT);

        AddParameterWindowController addParameterWindowController = new AddParameterWindowController(addNodeParameterStage);
        FXMLLoader loader = new FXMLLoader(App.class.getClassLoader().getResource("view/addParameterWindow.fxml"));
        loader.setController(addParameterWindowController);

        Scene scene = new Scene(loader.load());
        scene.setFill(Color.TRANSPARENT);
        addNodeParameterStage.setScene(scene);
        addParameterWindowController.loadData(node);

        addNodeParameterStage.showAndWait();

        ((Pane)previousPane.getRoot()).getChildren().remove(dimmedBackground);

        if (addParameterWindowController.getSelected()){
            return Optional.of(addParameterWindowController.getNodeParameter());
        }
        return Optional.empty();
    }

    public static Module getModule() throws IOException {
        Scene previousPane = App.getScene();

        Rectangle dimmedBackground = new Rectangle(App.getScene().getWidth(), App.getScene().getHeight());
        dimmedBackground.setFill(Color.rgb(116,118,120,0.6));

        ((Pane)previousPane.getRoot()).getChildren().add(dimmedBackground);

        Stage selectModuleStage = new Stage();
        selectModuleStage.initModality(Modality.APPLICATION_MODAL);
        selectModuleStage.initStyle(StageStyle.TRANSPARENT);

        SelectModuleWindowController selectModuleWindowController = new SelectModuleWindowController(selectModuleStage);
        FXMLLoader loader = new FXMLLoader(App.class.getClassLoader().getResource("view/selectModuleWindow.fxml"));
        loader.setController(selectModuleWindowController);

        Scene scene = new Scene(loader.load());
        scene.setFill(Color.TRANSPARENT);
        selectModuleStage.setScene(scene);
        selectModuleStage.showAndWait();

        ((Pane)previousPane.getRoot()).getChildren().remove(dimmedBackground);

        return selectModuleWindowController.getSelectedModule();
    }

    public static void showModuleParameters(String moduleName) throws IOException {
        Scene previousPane = App.getScene();

        Rectangle dimmedBackground = new Rectangle(App.getScene().getWidth(), App.getScene().getHeight());
        dimmedBackground.setFill(Color.rgb(116,118,120,0.6));

        ((Pane)previousPane.getRoot()).getChildren().add(dimmedBackground);

        Stage selectModuleStage = new Stage();
        selectModuleStage.initModality(Modality.APPLICATION_MODAL);
        selectModuleStage.initStyle(StageStyle.TRANSPARENT);

        ShowModuleParameterWindowController showModuleParameterWindowController = new ShowModuleParameterWindowController(selectModuleStage);
        FXMLLoader loader = new FXMLLoader(App.class.getClassLoader().getResource("view/showModuleParameterWindow.fxml"));
        loader.setController(showModuleParameterWindowController);

        Scene scene = new Scene(loader.load());
        scene.setFill(Color.TRANSPARENT);
        selectModuleStage.setScene(scene);
        showModuleParameterWindowController.loadData(moduleName);
        selectModuleStage.showAndWait();

        ((Pane)previousPane.getRoot()).getChildren().remove(dimmedBackground);
    }


    private static boolean showChoiceWindow(String primaryText, String secondaryText) throws IOException {
        Scene previousPane = App.getScene();

        Rectangle dimmedBackground = new Rectangle(App.getScene().getWidth(), App.getScene().getHeight());
        dimmedBackground.setFill(Color.rgb(116,118,120,0.6));

        ((Pane)previousPane.getRoot()).getChildren().add(dimmedBackground);

        Stage choiceStage = new Stage();
        choiceStage.initModality(Modality.APPLICATION_MODAL);
        choiceStage.initStyle(StageStyle.TRANSPARENT);

        ChoiceWindowController choiceWindowController = new ChoiceWindowController(choiceStage);
        FXMLLoader loader = new FXMLLoader(App.class.getClassLoader().getResource("view/choiceWindow.fxml"));
        loader.setController(choiceWindowController);

        Scene scene = new Scene(loader.load());
        scene.setFill(Color.TRANSPARENT);
        choiceStage.setScene(scene);
        choiceWindowController.loadData(primaryText, secondaryText);
        choiceStage.showAndWait();

        ((Pane)previousPane.getRoot()).getChildren().remove(dimmedBackground);

        return choiceWindowController.getConfirmed();
    }

    private static void showInformationWindow(String primaryText, String secondaryText) throws IOException {
        Scene previousPane = App.getScene();

        Rectangle dimmedBackground = new Rectangle(App.getScene().getWidth(), App.getScene().getHeight());
        dimmedBackground.setFill(Color.rgb(116,118,120,0.6));

        ((Pane)previousPane.getRoot()).getChildren().add(dimmedBackground);

        Stage informationStage = new Stage();
        informationStage.initModality(Modality.APPLICATION_MODAL);
        informationStage.initStyle(StageStyle.TRANSPARENT);

        InformationWindowController informationWindowController = new InformationWindowController(informationStage);
        FXMLLoader loader = new FXMLLoader(App.class.getClassLoader().getResource("view/informationWindow.fxml"));
        loader.setController(informationWindowController);

        Scene scene = new Scene(loader.load());
        scene.setFill(Color.TRANSPARENT);
        informationStage.setScene(scene);
        informationWindowController.loadData(primaryText, secondaryText);
        informationStage.showAndWait();

        ((Pane)previousPane.getRoot()).getChildren().remove(dimmedBackground);
    }
}
