/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.concurrent.Task;

import javax.net.ssl.SSLHandshakeException;
import java.io.IOException;
import java.io.InputStream;
import java.net.Authenticator;
import java.net.PasswordAuthentication;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.time.Duration;
import java.util.Properties;

public class RequestTagsTask extends Task<ObservableList<String>> {

    private final String repositoryName;
    private Properties properties;

    public RequestTagsTask(String repositoryName) {
        this.repositoryName = repositoryName;

        try (InputStream input = BuildingImageTask.class.getClassLoader().getResourceAsStream("config.properties")) {

            properties = new Properties();

            if (input == null) {
                System.out.println("Sorry, unable to find configuration file.");
                return;
            }
            properties.load(input);

        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    @Override
    protected ObservableList<String> call() throws Exception {

        HttpClient client = HttpClient.newBuilder()
                .authenticator(new Authenticator(){
                    @Override
                    protected PasswordAuthentication getPasswordAuthentication() {
                        return new PasswordAuthentication(
                                properties.getProperty("docker.registry.username"),
                                properties.getProperty("docker.registry.password").toCharArray());
                    }
                })
                .connectTimeout(Duration.ofSeconds(30))
                .build();

        try {
            HttpRequest request = HttpRequest.newBuilder()
                    .GET()
                    .uri(URI.create("https://orchestrator:443/v2/" + repositoryName +"/tags/list"))
                    .build();
            HttpResponse<String> response = client.send(request, HttpResponse.BodyHandlers.ofString());
            ObjectMapper objectMapper = new ObjectMapper();
            JsonNode jsonNode = objectMapper.readTree(response.body());
            if(!jsonNode.get("tags").isNull()){
                ObservableList<String> list = FXCollections.observableArrayList();
                for(JsonNode node : jsonNode.get("tags")){
                    list.add(node.asText());
                }
                return list;
            }
        }catch(SSLHandshakeException e){
            //
        }
        return null;
    }
}
