/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.PopUpWindows;

import com.jfoenix.controls.JFXProgressBar;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.RequestTask;
import javafx.beans.property.ReadOnlyObjectWrapper;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableRow;
import javafx.scene.control.TableView;
import javafx.stage.Stage;

import java.io.IOException;

public class SelectRepositoryWindowController {

    @FXML
    private TableView<String> table;

    @FXML
    private TableColumn<String, String> nameCol;

    private Stage stage;

    private String repositoryName;
    private String tag;

    public SelectRepositoryWindowController(Stage stage) {
        this.stage = stage;
    }

    public void initialize(){
        nameCol.setCellValueFactory(data -> new ReadOnlyObjectWrapper<>(data.getValue()));
        RequestTask task = new RequestTask();
        Thread thread = new Thread(task);
        thread.setDaemon(true);
        thread.start();
        table.setPlaceholder(new JFXProgressBar());

        task.setOnSucceeded(e->{
            table.getItems().addAll(task.getValue());
        });

        table.setRowFactory( tv -> {
            TableRow<String> row = new TableRow<>();
            row.setOnMouseClicked(event -> {
                if (event.getClickCount() == 2 && (! row.isEmpty()) ) {
                    try {
                        showTags();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            });
            return row ;
        });
    }

    public String getRepositoryName(){
        return repositoryName;
    }

    public String getTag(){
        return tag;
    }

    public void setTag(String tag) {
        this.tag = tag;
    }

    private void showTags() throws IOException {

        repositoryName = table.getSelectionModel().getSelectedItem();

        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/selectTagWindow.fxml"));
        SelectTagWindowController selectTagWindowController =  new SelectTagWindowController(stage, this);
        loader.setController(selectTagWindowController);
        stage.setScene(new Scene(loader.load()));
        selectTagWindowController.loadData(repositoryName);

    }

    public void close(){
        stage.close();
    }
}
