/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.PopUpWindows;

import com.jfoenix.controls.JFXProgressBar;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.RequestTagsTask;
import javafx.beans.property.ReadOnlyObjectWrapper;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableRow;
import javafx.scene.control.TableView;
import javafx.stage.Stage;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;

import java.io.IOException;

public class SelectTagWindowController {

    @FXML
    private TableView<String> table;

    @FXML
    private TableColumn<String, String> nameCol;

    @FXML
    private Label repositoryNameLabel;

    private Stage stage;
    private SelectRepositoryWindowController selectRepositoryWindowController;

    public SelectTagWindowController(Stage stage, SelectRepositoryWindowController selectRepositoryWindowController) {
        this.stage = stage;
        this.selectRepositoryWindowController = selectRepositoryWindowController;
    }

    public void initialize(){
        nameCol.setCellValueFactory(data -> new ReadOnlyObjectWrapper<>(data.getValue()));

        table.setRowFactory( tv -> {
            TableRow<String> row = new TableRow<>();
            row.setOnMouseClicked(event -> {
                if (event.getClickCount() == 2 && (! row.isEmpty()) ) {
                    selectRepositoryWindowController.setTag(row.getItem());
                    stage.close();
                }
            });
            return row ;
        });
    }

    public void close(){
        stage.close();
    }

    public void back() throws IOException {
        SelectRepositoryWindowController selectRepositoryWindowController = new SelectRepositoryWindowController(stage);

        FXMLLoader loader = new FXMLLoader(App.class.getClassLoader().getResource("view/selectRepositoryWindow.fxml"));
        loader.setController(selectRepositoryWindowController);
        Scene scene = new Scene(loader.load());
        stage.setScene(scene);
    }

    public void loadData(String repositoryName) {
        RequestTagsTask task = new RequestTagsTask(repositoryName);
        Thread thread = new Thread(task);
        thread.setDaemon(true);
        thread.start();
        table.setPlaceholder(new JFXProgressBar());

        task.setOnSucceeded(e->{
            if(task.getValue() != null) {
                table.getItems().addAll(task.getValue());
            }
            else {
                table.setPlaceholder(new Label("no data"));
            }
        });
        task.setOnFailed(e ->{
            table.setPlaceholder(new Label("Unable to load data."));
        });
    }
}
