/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.images;

import com.jfoenix.controls.JFXProgressBar;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.BuildingImageTask;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.BaseController;
import javafx.beans.property.ReadOnlyObjectWrapper;
import javafx.beans.value.ChangeListener;
import javafx.concurrent.Task;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;

public class ViewBuildingImagesController extends BaseController {

    @FXML
    private TableView<BuildingImageTask> table;

    @FXML
    private TableColumn<BuildingImageTask, String> imageNameCol;

    @FXML
    private TableColumn<BuildingImageTask, Task.State> statusCol;

    @FXML
    private TableColumn<BuildingImageTask, Double> progressCol;

    @FXML
    private TableColumn<BuildingImageTask, BuildingImageTask> cancelCol;

    @FXML
    protected ToggleButton imagesButton;
    @FXML
    protected ToggleButton buildingImagesButton;

    public void initialize(){
        ToggleGroup group2 = new ToggleGroup();
        imagesButton.setToggleGroup(group2);
        buildingImagesButton.setToggleGroup(group2);
        buildingImagesButton.setSelected(true);

        group2.selectedToggleProperty().addListener((obsVal, oldVal, newVal) -> {
            if (newVal == null)
                oldVal.setSelected(true);
        });

        table.setPlaceholder(new Label("No images are currently being built."));

        imageNameCol.setCellValueFactory(data -> new ReadOnlyObjectWrapper<>(data.getValue().getImageName()));
        statusCol.setCellValueFactory(data -> data.getValue().stateProperty());
        progressCol.setCellValueFactory(data -> data.getValue().progressProperty().asObject());
        progressCol.setCellFactory(c -> new ProgressTableCell());
        cancelCol.setCellValueFactory(data -> new ReadOnlyObjectWrapper<>(data.getValue()));
        cancelCol.setCellFactory(c -> new ButtonTableCell());

        table.getItems().addAll(App.getRunningTasks());
    }

    private static class ButtonTableCell extends TableCell<BuildingImageTask, BuildingImageTask> {
        Button cancelButton = new Button("Cancel");
        {
            cancelButton.getStyleClass().add("defaultButton");
            cancelButton.setOnAction(e -> getItem().cancel());

            ChangeListener<Boolean> disableListener = (obs, wasRunning, isNowRunning) ->
                    cancelButton.setDisable(!isNowRunning);

            this.itemProperty().addListener((obs, oldTask, newTask) -> {
                if (oldTask != null) {
                    oldTask.runningProperty().removeListener(disableListener);
                }
                if (newTask == null) {
                    this.setGraphic(null);
                } else {
                    this.setGraphic(cancelButton);
                    cancelButton.setDisable(!newTask.isRunning());
                    newTask.runningProperty().addListener(disableListener);
                }
            });

        }

        @Override
        public void updateItem(BuildingImageTask value, boolean empty) {
            super.updateItem(value, empty);
            if (empty || value == null) {
                setGraphic(null);
            } else {
                setGraphic(cancelButton);
            }
        }
    }

    private static class ProgressTableCell extends TableCell<BuildingImageTask, Double> {
        final JFXProgressBar progressBar = new JFXProgressBar();

        @Override
        public void updateItem(Double value, boolean empty) {
            if (empty || value == null) {
                setGraphic(null);
            } else {
                setGraphic(progressBar);
                progressBar.setProgress(value);
            }
        }
    }
}
