/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.images;

import com.jfoenix.controls.JFXProgressBar;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.RequestTask;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.BaseController;
import javafx.beans.property.ReadOnlyObjectWrapper;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.*;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;

import java.io.IOException;

public class ViewImagesController extends BaseController {

    @FXML
    private TableView<String> table;
    @FXML
    private TableColumn<String, String> repositoryNameCol;

    @FXML
    protected ToggleButton imagesButton;
    @FXML
    protected ToggleButton buildingImagesButton;


    public void initialize() {
        ToggleGroup group2 = new ToggleGroup();
        imagesButton.setToggleGroup(group2);
        buildingImagesButton.setToggleGroup(group2);
        imagesButton.setSelected(true);

        group2.selectedToggleProperty().addListener((obsVal, oldVal, newVal) -> {
            if (newVal == null)
                oldVal.setSelected(true);
        });

        repositoryNameCol.setCellValueFactory(data -> new ReadOnlyObjectWrapper<>(data.getValue()));

        RequestTask task = new RequestTask();
        Thread thread = new Thread(task);
        thread.setDaemon(true);
        thread.start();
        table.setPlaceholder(new JFXProgressBar());

        task.setOnSucceeded(e->{
            table.getItems().addAll(task.getValue());
        });
        task.setOnFailed(e ->{
            table.setPlaceholder(new Label("Unable to load data."));
        });

        table.setRowFactory( (tableView) -> new SetTableRow(() -> {
                try {
                    showTags();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }).call(tableView));
    }

    public void showTags() throws IOException {
        String repositoryName = table.getSelectionModel().getSelectedItem();

        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewTags.fxml"));
        App.getScene().setRoot(loader.load());
        loader.<ViewTagsController>getController().loadData(repositoryName);
    }

    public void viewBuildingImages() throws IOException {
        App.setRoot("viewBuildingImages");
    }
    public void addImage() throws IOException {
        App.setRoot("addImage");
    }
}

