/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.images;

import com.jfoenix.controls.JFXProgressBar;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.DeleteTagTask;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.PopUpWindow;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.RequestTagsTask;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.BaseController;
import javafx.beans.property.ReadOnlyObjectWrapper;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;

import java.io.IOException;

public class ViewTagsController extends BaseController {

    @FXML
    private Label repositoryNameLabel;

    @FXML
    private TableView<String> table;

    @FXML
    private TableColumn<String, String> tagCol;

    @FXML
    private Button deleteButton;

    public void initialize(){
        tagCol.setCellValueFactory(data -> new ReadOnlyObjectWrapper<>(data.getValue()));
        deleteButton.visibleProperty().bind(table.getSelectionModel().selectedItemProperty().isNotNull());
    }


    public void loadData(String repositoryName) {
        repositoryNameLabel.setText(repositoryNameLabel.getText() +  repositoryName);

        RequestTagsTask task = new RequestTagsTask(repositoryName);
        Thread thread = new Thread(task);
        thread.setDaemon(true);
        thread.start();
        table.setPlaceholder(new JFXProgressBar());

        task.setOnSucceeded(e->{
            if(task.getValue() != null) {
                table.getItems().addAll(task.getValue());
            }
            else {
                table.setPlaceholder(new Label("no data"));
            }
        });
        task.setOnFailed(e ->{
            table.setPlaceholder(new Label("Unable to load data."));
        });
    }

    public void delete() throws IOException {
        if(PopUpWindow.getModuleDeleteResult()){
            String tagToDelete = table.getSelectionModel().getSelectedItem();

            DeleteTagTask task = new DeleteTagTask(repositoryNameLabel.getText(), tagToDelete);
            Thread thread = new Thread(task);
            thread.setDaemon(true);
            //thread.start();
            table.setPlaceholder(new JFXProgressBar());

            table.getItems().remove(tagToDelete);
            table.refresh();
        }
    }
}
