/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules;

import com.jfoenix.controls.JFXRadioButton;
import com.jfoenix.controls.JFXTextField;
import com.jfoenix.validation.RegexValidator;
import com.jfoenix.validation.RequiredFieldValidator;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.InternalizationUtility;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.PopUpWindow;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.ModuleService;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToggleGroup;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;

import java.io.IOException;
import java.net.URL;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class AddModuleController implements Initializable {

    private ModuleService moduleService = new ModuleService();

    @FXML
    private JFXTextField nameField;
    @FXML
    private JFXTextField typeField;
    @FXML
    private JFXTextField healthPortField;
    @FXML
    private JFXTextField imageField;

    @FXML
    private JFXRadioButton isBlocking;
    @FXML
    private JFXRadioButton isNotBlocking;

    @FXML
    private ToggleButton englishLanguageButton;
    @FXML
    private ToggleButton czechLanguageButton;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        ToggleGroup group = new ToggleGroup();
        englishLanguageButton.setToggleGroup(group);
        czechLanguageButton.setToggleGroup(group);

        if (InternalizationUtility.getLocale() == Locale.ENGLISH){
            englishLanguageButton.setSelected(true);
        } else {
            czechLanguageButton.setSelected(true);
        }

        ToggleGroup group2 = new ToggleGroup();
        isBlocking.setToggleGroup(group2);
        isNotBlocking.setToggleGroup(group2);
        isBlocking.setSelected(true);

        RequiredFieldValidator requiredFieldValidator = new RequiredFieldValidator();
        requiredFieldValidator.setMessage("Module's name is required");
        nameField.getValidators().add(requiredFieldValidator);

        nameField.textProperty().addListener((x, oldValue, newValue) -> {
            if (newValue != null){
                nameField.validate();
            }
        });

        RegexValidator portNumberValidator = new RegexValidator();
        portNumberValidator.setMessage("Must be a valid port number. [0 - 65535]");
        portNumberValidator.setRegexPattern("^$|^([1-9]|[1-5]?[0-9]{2,4}|6[1-4][0-9]{3}|65[1-4][0-9]{2}|655[1-2][0-9]|6553[1-5])$");
        healthPortField.getValidators().add(portNumberValidator);

        healthPortField.textProperty().addListener((x, old, news) -> {
            if (news != null){
                healthPortField.validate();
            }
        });
    }

    public void changeToEnglishLanguage() throws IOException {
        InternalizationUtility.setLocale(Locale.ENGLISH);
        App.setRoot("addModule");
    }

    public void changeToCzechLanguage() throws IOException {
        InternalizationUtility.setLocale(new Locale("cs"));
        App.setRoot("addModule");
    }

    public void viewModules() throws IOException {
        App.setRoot("viewModules");
    }

    public void viewOperations() throws IOException {
        App.setRoot("viewOperations");
    }

    public void chooseImage() throws IOException {
        List<String> result = PopUpWindow.showRepositories();
        if (result.get(0) != null && result.get(1) != null){
            imageField.setText(result.get(0) + ":" + result.get(1));
        }
    }

    @FXML
    private void save() throws IOException {
        if (nameField.getText().equals("")){
            PopUpWindow.showRequiredNameWindow();
            return;
        }

        if(moduleService.getModuleByName(nameField.getText()).size() != 0){
            PopUpWindow.showDuplicateNameWindow();
            return;
        }

        String regex = "^$|^([1-9]|[1-5]?[0-9]{2,4}|6[1-4][0-9]{3}|65[1-4][0-9]{2}|655[1-2][0-9]|6553[1-5])$";
        if (!Pattern.matches(regex, healthPortField.getText())){
            PopUpWindow.showInvalidPortNumberWindow();
            return;
        }

        Module module = new Module();
        module.setName(nameField.getText());
        module.setType(typeField.getText());
        if (!healthPortField.getText().isEmpty()){
            module.setHealthPort(Integer.parseInt(healthPortField.getText()));
        }

        Pattern pattern=Pattern.compile("(.*):(.*)");
        Matcher matcher=pattern.matcher(imageField.getText());

        String nameStr="";
        String tagString="";
        if(matcher.find())
        {
            nameStr=matcher.group(1);
            tagString=matcher.group(2);
        }

        module.setImage(nameStr);
        module.setTag(tagString);
        module.setIsBlocking(isBlocking.isSelected());

        moduleService.addModule(module);

        viewModules();
    }
}
