/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules;

import com.jfoenix.controls.JFXRadioButton;
import com.jfoenix.controls.JFXTextField;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.InternalizationUtility;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.PopUpWindow;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.BaseController;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.ModuleParameterService;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.Label;
import javafx.scene.control.ToggleGroup;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.ModuleParameter;

import java.io.IOException;
import java.util.ResourceBundle;

public class AddModulesParameterController extends BaseController {

    private ModuleParameterService moduleParameterService = new ModuleParameterService();

    private Module module;

    @FXML
    private Label nameLabel;
    @FXML
    private Label typeLabel;
    @FXML
    private Label isBlockingLabel;
    @FXML
    private Label healthPortLabel;
    @FXML
    private Label imageLabel;
    @FXML
    private Label tagLabel;

    @FXML
    private JFXTextField nameField;
    @FXML
    private JFXTextField defaultValueField;
    @FXML
    private JFXTextField descriptionField;
    @FXML
    private JFXRadioButton isStrict;
    @FXML
    private JFXRadioButton isNotStrict;

    @FXML
    private void initialize() {
        ToggleGroup isStrictGroup = new ToggleGroup();
        isStrict.setToggleGroup(isStrictGroup);
        isNotStrict.setToggleGroup(isStrictGroup);
        isStrict.setSelected(true);
    }

    public void showDetails() throws IOException{
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesPorts.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);
    }

    public void save() throws IOException {

        if (nameField.getText().equals("")){
            PopUpWindow.showRequiredNameWindow();
            return;
        }

        if(moduleParameterService.getModuleParametersByName(nameField.getText()).size() != 0){
            PopUpWindow.showDuplicateNameWindow();
            return;
        }

        ModuleParameter moduleParameter = new ModuleParameter();
        moduleParameter.setName(nameField.getText());
        moduleParameter.setDefaultValue(defaultValueField.getText());
        moduleParameter.setDescription(descriptionField.getText());
        moduleParameter.setIsStrict(isStrict.isSelected());
        moduleParameter.setModule(module);

        moduleParameterService.addModuleParameter(moduleParameter);

        showDetails();
    }

    public void loadData(Module module) {
        this.module = module;
        nameLabel.setText(module.getName());
        typeLabel.setText(module.getType());
        isBlockingLabel.setText(String.valueOf(module.getIsBlocking()));
        healthPortLabel.setText(String.valueOf(module.getHealthPort()));
        imageLabel.setText(module.getImage());
        tagLabel.setText(module.getTag());
    }
}