/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules;

import com.jfoenix.controls.JFXRadioButton;
import com.jfoenix.controls.JFXTextField;
import com.jfoenix.validation.RegexValidator;
import com.jfoenix.validation.RequiredFieldValidator;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.InternalizationUtility;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.PopUpWindow;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.BaseController;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.ModulePort;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.ModulePortService;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.Label;
import javafx.scene.control.ToggleGroup;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;

import java.io.IOException;
import java.util.ResourceBundle;
import java.util.regex.Pattern;

public class AddModulesPortController extends BaseController {

    private ModulePortService modulePortService = new ModulePortService();

    private Module module;

    @FXML
    private Label nameLabel;
    @FXML
    private Label typeLabel;
    @FXML
    private Label isBlockingLabel;
    @FXML
    private Label healthPortLabel;
    @FXML
    private Label imageLabel;
    @FXML
    private Label tagLabel;

    @FXML
    private JFXTextField nameField;
    @FXML
    private JFXTextField numberField;
    @FXML
    private JFXRadioButton isPublic;
    @FXML
    private JFXRadioButton isNotPublic;

    @FXML
    private void initialize(){
        ToggleGroup isPublicGroup = new ToggleGroup();
        isPublic.setToggleGroup(isPublicGroup);
        isNotPublic.setToggleGroup(isPublicGroup);
        isPublic.setSelected(true);

        RequiredFieldValidator requiredFieldValidator = new RequiredFieldValidator();
        requiredFieldValidator.setMessage("Ports's name is required");
        nameField.getValidators().add(requiredFieldValidator);

        nameField.textProperty().addListener((x, oldValue, newValue) -> {
            if (newValue != null){
                nameField.validate();
            }
        });

        RegexValidator portNumberValidator = new RegexValidator();
        portNumberValidator.setMessage("Must be a valid port number. [0 - 65535]");
        portNumberValidator.setRegexPattern("^([1-9]|[1-5]?[0-9]{2,4}|6[1-4][0-9]{3}|65[1-4][0-9]{2}|655[1-2][0-9]|6553[1-5])$");
        numberField.getValidators().add(portNumberValidator);

        numberField.textProperty().addListener((x, old, news) -> {
            if (news != null){
                numberField.validate();
            }
        });
    }

    public void showDetails() throws IOException {
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesPorts.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);
    }

    @FXML
    private void save() throws IOException {
        if (nameField.getText().equals("")){
            PopUpWindow.showRequiredNameWindow();
            return;
        }

        if(modulePortService.getModulePortsByName(nameField.getText()).size() != 0){
            PopUpWindow.showDuplicateNameWindow();
            return;
        }

        String regex = "^([1-9]|[1-5]?[0-9]{2,4}|6[1-4][0-9]{3}|65[1-4][0-9]{2}|655[1-2][0-9]|6553[1-5])$";
        if (!Pattern.matches(regex, numberField.getText())){
            PopUpWindow.showInvalidPortNumberWindow();
            return;
        }

        if(modulePortService.getModulePortsByNumber(Integer.parseInt(numberField.getText())).size() != 0){
            PopUpWindow.showDuplicatePortNumberWindow();
            return;
        }

        ModulePort modulePort = new ModulePort();
        modulePort.setName(nameField.getText());
        modulePort.setNumber(Integer.parseInt(numberField.getText()));
        modulePort.setIsPublic(isPublic.isSelected());
        modulePort.setModule(module);

        modulePortService.addModulePort(modulePort);
        showDetails();
    }

    public void loadData(Module module) {
        this.module = module;
        nameLabel.setText(module.getName());
        typeLabel.setText(module.getType());
        isBlockingLabel.setText(String.valueOf(module.getIsBlocking()));
        healthPortLabel.setText(String.valueOf(module.getHealthPort()));
        imageLabel.setText(module.getImage());
        tagLabel.setText(module.getTag());
    }
}
