/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules;

import com.jfoenix.controls.JFXRadioButton;
import com.jfoenix.controls.JFXTextField;
import com.jfoenix.validation.RegexValidator;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.PopUpWindow;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.ModuleService;
import javafx.fxml.FXML;
import javafx.scene.control.ToggleGroup;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;

import java.io.File;
import java.io.IOException;
import java.util.regex.Pattern;


public class UpdateModuleController {

    private ModuleService moduleService = new ModuleService();

    private Module module;

    @FXML
    private JFXTextField nameField;
    @FXML
    private JFXTextField typeField;
    @FXML
    private JFXTextField healthPortField;
    @FXML
    private JFXTextField imageField;
    @FXML
    private JFXTextField tagField;
    @FXML
    private JFXRadioButton isBlocking;
    @FXML
    private JFXRadioButton isNotBlocking;

    @FXML
    public void initialize(){
        ToggleGroup group = new ToggleGroup();
        isBlocking.setToggleGroup(group);
        isNotBlocking.setToggleGroup(group);

        RegexValidator portNumberValidator = new RegexValidator();
        portNumberValidator.setMessage("Must be an valid port. [0 - 65535]");
        portNumberValidator.setRegexPattern("^([1-9]|[1-5]?[0-9]{2,4}|6[1-4][0-9]{3}|65[1-4][0-9]{2}|655[1-2][0-9]|6553[1-5])$");
        healthPortField.getValidators().add(portNumberValidator);

        healthPortField.textProperty().addListener((x, old, news) -> {
            if (news != null){
                healthPortField.validate();
            }
        });
    }

    public void viewModules() throws IOException {
        App.setRoot("viewModules");
    }

    public void viewOperations() throws IOException {
        App.setRoot("viewOperations");
    }

    public void chooseFile(){
        Stage stage = new Stage();
        FileChooser fileChooser = new FileChooser();
        File chosenFile =  fileChooser.showOpenDialog(stage);
        if (chosenFile != null) {
            imageField.setText(chosenFile.getName());
        }
    }

    @FXML
    private void update() throws IOException {

        String regex = "^([1-9]|[1-5]?[0-9]{2,4}|6[1-4][0-9]{3}|65[1-4][0-9]{2}|655[1-2][0-9]|6553[1-5])$";
        if (!Pattern.matches(regex, healthPortField.getText())){
            PopUpWindow.showInvalidPortNumberWindow();
            return;
        }

        module.setName(nameField.getText());
        module.setType(typeField.getText());
        module.setHealthPort(Integer.parseInt(healthPortField.getText()));
        module.setImage(imageField.getText());
        module.setTag(tagField.getText());
        module.setIsBlocking(isBlocking.isSelected());

        moduleService.updateModule(module);
        viewModules();
    }

    public void loadData(Module module) {
        this.module = module;
        nameField.setText(module.getName());
        typeField.setText(module.getType());
        isBlocking.setSelected(module.getIsBlocking());
        isNotBlocking.setSelected(!module.getIsBlocking());
        healthPortField.setText(String.valueOf(module.getHealthPort()));
        imageField.setText(module.getImage());
        tagField.setText(module.getTag());
    }

}
