/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules;

import com.jfoenix.controls.JFXRadioButton;
import com.jfoenix.controls.JFXTextField;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.ModuleParameterService;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.Label;
import javafx.scene.control.ToggleGroup;
import javafx.scene.layout.AnchorPane;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.ModuleParameter;

import java.io.IOException;

public class UpdateModulesParameterController {

    private ModuleParameterService modulePortService = new ModuleParameterService();

    private Module module;

    private ModuleParameter moduleParameter;

    @FXML
    private AnchorPane infoPane;
    @FXML
    private AnchorPane attributesPane;
    @FXML
    private AnchorPane valuesPane;

    @FXML
    private Label nameLabel;
    @FXML
    private Label typeLabel;
    @FXML
    private Label requiredLabel;
    @FXML
    private Label healthPortLabel;
    @FXML
    private Label imageLabel;
    @FXML
    private Label tagLabel;

    @FXML
    private JFXTextField name;
    @FXML
    private JFXTextField defaultValue;
    @FXML
    private JFXRadioButton overridable;
    @FXML
    private JFXRadioButton notOverridable;
    @FXML
    private JFXRadioButton possiblyDifferent;
    @FXML
    private JFXRadioButton possiblyNotDifferent;

    @FXML
    public void initialize() {
        infoPane.widthProperty().addListener((obs, oldWidth, newWidth) -> {
            attributesPane.setPrefWidth((Double) newWidth / 2);
            valuesPane.setPrefWidth((Double) newWidth / 2);
        });

        ToggleGroup overridableGroup = new ToggleGroup();
        overridable.setToggleGroup(overridableGroup);
        notOverridable.setToggleGroup(overridableGroup);

        ToggleGroup possiblyDifferentGroup = new ToggleGroup();
        possiblyDifferent.setToggleGroup(possiblyDifferentGroup);
        possiblyNotDifferent.setToggleGroup(possiblyDifferentGroup);
    }

    @FXML
    private void switchToViewModules() throws IOException {
        App.setRoot("viewModules");
    }

    @FXML
    private void switchToAddModule() throws IOException {
        App.setRoot("addModule");
    }

    private void switchToViewModulesParameters() throws IOException {
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesParameters.fxml"));
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesParametersController>getController().loadData(module);
    }

    @FXML
    private void update() throws IOException {

        moduleParameter.setDefaultValue(defaultValue.getText());
        //moduleParameter.setDescription(overridable.isSelected());
        moduleParameter.setIsStrict(possiblyDifferent.isSelected());

        modulePortService.updateModuleParameter(moduleParameter);

        switchToViewModulesParameters();
    }

    public void loadData(Module module, ModuleParameter moduleParameter) {
        this.module = module;
        nameLabel.setText(module.getName());
        typeLabel.setText(module.getType());
        requiredLabel.setText(String.valueOf(module.getIsBlocking()));
        healthPortLabel.setText(String.valueOf(module.getHealthPort()));
        imageLabel.setText(module.getImage());
        tagLabel.setText(module.getTag());

        this.moduleParameter = moduleParameter;
        name.setText(moduleParameter.getName());
        defaultValue.setText(moduleParameter.getDefaultValue());
        //overridable.setSelected(moduleParameter.getDescription());
        //notOverridable.setSelected(!moduleParameter.getDescription());
        possiblyDifferent.setSelected(moduleParameter.getIsStrict());
        possiblyNotDifferent.setSelected(!moduleParameter.getIsStrict());
    }
}