/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules;

import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.InternalizationUtility;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.PopUpWindow;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.BaseController;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.ModuleService;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.collections.transformation.FilteredList;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.control.*;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.layout.HBox;

import java.io.IOException;
import java.net.URL;
import java.util.Locale;
import java.util.ResourceBundle;

public class ViewModulesController extends BaseController implements Initializable{

    private final ModuleService moduleService = new ModuleService();

    @FXML
    private TableView<Module> table;

    @FXML
    private Button deleteButton;

    @FXML
    private Button updateButton;

    @FXML
    private Button showDetailsButton;

    @FXML
    private TableColumn<Module,String> nameCol;

    @FXML
    private TableColumn<Module,String> typeCol;

    @FXML
    private TableColumn<Module,Integer> healthPortCol;

    @FXML
    private ToggleButton englishLanguageButton;
    @FXML
    private ToggleButton czechLanguageButton;

    @FXML
    private TextField searchField;

    @FXML
    private HBox hbox;

    @Override
    public void initialize(URL location, ResourceBundle resources) {

        hbox.setVisible(false);

        ToggleGroup group = new ToggleGroup();
        englishLanguageButton.setToggleGroup(group);
        czechLanguageButton.setToggleGroup(group);

        if (InternalizationUtility.getLocale() == Locale.ENGLISH){
            englishLanguageButton.setSelected(true);
        } else {
            czechLanguageButton.setSelected(true);
        }

        group.selectedToggleProperty().addListener((obsVal, oldVal, newVal) -> {
            if (newVal == null)
                oldVal.setSelected(true);
        });

        deleteButton.visibleProperty().bind(table.getSelectionModel().selectedItemProperty().isNotNull());
        showDetailsButton.visibleProperty().bind(table.getSelectionModel().selectedItemProperty().isNotNull());
        updateButton.visibleProperty().bind(table.getSelectionModel().selectedItemProperty().isNotNull());

        nameCol.setCellValueFactory(new PropertyValueFactory<>("name"));
        typeCol.setCellValueFactory(new PropertyValueFactory<>("type"));
        healthPortCol.setCellValueFactory(new PropertyValueFactory<>("healthPort"));

        table.setRowFactory( tv -> {
            TableRow<Module> row = new TableRow<>();
            row.setOnMouseClicked(event -> {
                if (event.getClickCount() == 2 && (! row.isEmpty()) ) {
                    try {
                        showDetails();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            });
            return row ;
        });

        //table.getItems().addAll(moduleService.getAllModules());
        ObservableList<Module> list = FXCollections.observableArrayList(moduleService.getAllModules());
        FilteredList<Module> filteredList = new FilteredList<>(list);
        table.setItems(filteredList);

        searchField.textProperty().addListener((obs, oldValue, newValue)->
                filteredList.setPredicate(x -> x.getName().contains(newValue))
        );
    }

    public void changeToEnglishLanguage() throws IOException {
        InternalizationUtility.setLocale(Locale.ENGLISH);
        App.setRoot("viewModules");
    }

    public void changeToCzechLanguage() throws IOException {
        InternalizationUtility.setLocale(new Locale("cs"));
        App.setRoot("viewModules");
    }
    public void addModule() throws IOException {

        App.setRoot("addModule");
    }

    public void showDetails() throws IOException{
        Module modules = table.getSelectionModel().getSelectedItem();

        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getResource("/view/viewModulesPorts.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(modules);
    }

    public void update() throws IOException {
        Module module = table.getSelectionModel().getSelectedItem();

        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/updateModule.fxml"));
        App.getScene().setRoot(loader.load());
        loader. <UpdateModuleController>getController().loadData(module);
    }

    public void delete() throws IOException {

        if(PopUpWindow.getModuleDeleteResult()){
            Module moduleToDelete = table.getSelectionModel().getSelectedItem();
            moduleService.deleteModule(moduleToDelete);
            //table.getItems().remove(moduleToDelete);
        }
    }
}