/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules;

import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.InternalizationUtility;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.PopUpWindow;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.BaseController;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.ModuleService;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.layout.HBox;

import java.io.IOException;
import java.net.URL;
import java.util.Locale;
import java.util.ResourceBundle;


public class ViewModulesDetailsController extends BaseController implements Initializable {

    protected Module module;

    protected ModuleService moduleService = new ModuleService();

    @FXML
    protected Label nameLabel;
    @FXML
    protected Label typeLabel;
    @FXML
    protected Label isBlockingLabel;
    @FXML
    protected Label healthPortLabel;
    @FXML
    protected Label imageLabel;
    @FXML
    protected Label tagLabel;

    @FXML
    protected ToggleButton parametersButton;
    @FXML
    protected ToggleButton portsButton;
    @FXML
    protected ToggleButton appearsInButton;

    @FXML
    protected ToggleButton englishLanguageButton;
    @FXML
    protected ToggleButton czechLanguageButton;

    @FXML
    protected HBox hbox;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        ToggleGroup group = new ToggleGroup();
        englishLanguageButton.setToggleGroup(group);
        czechLanguageButton.setToggleGroup(group);

        hbox.setVisible(false);

        if (InternalizationUtility.getLocale() == Locale.ENGLISH){
            englishLanguageButton.setSelected(true);
        } else {
            czechLanguageButton.setSelected(true);
        }

        ToggleGroup group2 = new ToggleGroup();
        portsButton.setToggleGroup(group2);
        parametersButton.setToggleGroup(group2);
        appearsInButton.setToggleGroup(group2);
    }

    public void viewModulesPorts() throws IOException {
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesPorts.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);
    }

    public void viewModulesParameters() throws IOException {
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesParameters.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);
    }

    public void viewModulesOperations() throws IOException {
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesOperations.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);
    }

    public void delete() throws IOException {
        if(PopUpWindow.getModuleDeleteResult()){
            moduleService.deleteModule(module);
            viewModules();
        }
    }

    public void update() throws IOException {
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/updateModule.fxml"));
        App.getScene().setRoot(loader.load());
        loader.<UpdateModuleController>getController().loadData(module);
    }

    public void loadData(Module module) {
        this.module = module;
        nameLabel.setText(module.getName());
        typeLabel.setText(module.getType());
        isBlockingLabel.setText(String.valueOf(module.getIsBlocking()));
        healthPortLabel.setText(String.valueOf(module.getHealthPort()));
        imageLabel.setText(module.getImage());
        tagLabel.setText(module.getTag());
    }
}
