/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules;

import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.InternalizationUtility;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.NodeService;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.*;
import javafx.scene.control.cell.PropertyValueFactory;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.operations.AddOperationController;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.ModuleParameter;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Operation;

import java.io.IOException;
import java.net.URL;
import java.util.Locale;
import java.util.ResourceBundle;

public class ViewModulesOperationsController extends ViewModulesDetailsController{

    private NodeService nodeService = new NodeService();

    @FXML
    private TableView<Operation> operationsTable;

    @FXML
    private TableColumn<ModuleParameter, String> nameCol;

    @FXML
    private Separator separator;

    @FXML
    private Button showDetailsButton;


    @Override
    public void initialize(URL location, ResourceBundle resources) {
        super.initialize(location, resources);

        appearsInButton.setSelected(true);

        operationsTable.setPlaceholder(new Label("There are no operations using this module."));

        nameCol.setCellValueFactory(new PropertyValueFactory<>("name"));

        operationsTable.setRowFactory( tv -> {
            TableRow<Operation> row = new TableRow<>();
            row.setOnMouseClicked(event -> {
                if (event.getClickCount() == 2 && (! row.isEmpty()) ) {
                    try {
                        showDetails();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            });
            return row ;
        });

        showDetailsButton.visibleProperty().bind(operationsTable.getSelectionModel().selectedItemProperty().isNotNull());
        separator.visibleProperty().bind(operationsTable.getSelectionModel().selectedItemProperty().isNotNull());
    }

    public void changeToEnglishLanguage() throws IOException {
        InternalizationUtility.setLocale(Locale.ENGLISH);
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesOperations.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);    }

    public void changeToCzechLanguage() throws IOException {
        InternalizationUtility.setLocale(new Locale("cs"));
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesOperations.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);
    }

    public void showDetails() throws IOException {
        Operation operation = operationsTable.getSelectionModel().getSelectedItem();

        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/addOperation.fxml"));
        App.getScene().setRoot(loader.load());
        loader.<AddOperationController>getController().loadData(operation);
    }

    public void loadData(Module module) {
        super.loadData(module);
        operationsTable.getItems().addAll(nodeService.getNodeByModuleName(module.getName()));
    }
}
