/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules;

import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.InternalizationUtility;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.ModuleParameter;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.ModuleParameterService;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.*;
import javafx.scene.control.cell.PropertyValueFactory;

import java.io.IOException;
import java.net.URL;
import java.util.Locale;
import java.util.ResourceBundle;

public class ViewModulesParametersController extends ViewModulesDetailsController{

    private ModuleParameterService moduleParameterService = new ModuleParameterService();

    @FXML
    private TableView<ModuleParameter> parametersTable;

    @FXML
    private TableColumn<ModuleParameter, String> nameCol;

    @FXML
    private TableColumn<ModuleParameter, String> defaultValueCol;

    @FXML
    private TableColumn<ModuleParameter, Boolean> isStrictCol;

    @FXML
    private TableColumn<ModuleParameter, String> descriptionCol;

    @FXML
    private Separator separator;

    @FXML
    private Button deleteParameterButton;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        super.initialize(location, resources);

        parametersButton.setSelected(true);

        parametersTable.setPlaceholder(new Label("No parameters available. Click on '+' to add new."));

        nameCol.setCellValueFactory(new PropertyValueFactory<>("name"));
        defaultValueCol.setCellValueFactory(new PropertyValueFactory<>("defaultValue"));
        descriptionCol.setCellValueFactory(new PropertyValueFactory<>("description"));
        isStrictCol.setCellValueFactory(new PropertyValueFactory<>("isStrict"));

        deleteParameterButton.visibleProperty().bind(parametersTable.getSelectionModel().selectedItemProperty().isNotNull());
        separator.visibleProperty().bind(parametersTable.getSelectionModel().selectedItemProperty().isNotNull());
    }

    public void addParameter() throws IOException {
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/addModulesParameter.fxml"));
        App.getScene().setRoot(loader.load());
        loader.<AddModulesParameterController>getController().loadData(module);
    }

    public void deleteParameter() {
        ModuleParameter moduleParameterToDelete = parametersTable.getSelectionModel().getSelectedItem();
        moduleParameterService.deleteModuleParameter(moduleParameterToDelete);
        parametersTable.getItems().remove(moduleParameterToDelete);
    }

    public void changeToEnglishLanguage() throws IOException {
        InternalizationUtility.setLocale(Locale.ENGLISH);
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesParameters.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);    }

    public void changeToCzechLanguage() throws IOException {
        InternalizationUtility.setLocale(new Locale("cs"));
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesParameters.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);
    }

    public void loadData(Module module) {
        super.loadData(module);
        parametersTable.getItems().addAll(moduleParameterService.getModuleParametersByModuleName(module.getName()));
    }
}
