/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules;

import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.InternalizationUtility;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.ModulePort;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.ModulePortService;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.*;
import javafx.scene.control.cell.PropertyValueFactory;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;

import java.io.IOException;
import java.net.URL;
import java.util.Locale;
import java.util.ResourceBundle;

public class ViewModulesPortsController extends ViewModulesDetailsController{

    private ModulePortService modulePortService = new ModulePortService();

    @FXML
    private TableView<ModulePort> portsTable;

    @FXML
    private TableColumn<ModulePort, String> portsNameCol;

    @FXML
    private TableColumn<ModulePort, String> numberCol;

    @FXML
    private TableColumn<ModulePort, Boolean> isPublicCol;

    @FXML
    private Separator separator;

    @FXML
    private Button deletePortButton;


    @Override
    public void initialize(URL location, ResourceBundle resources) {
        super.initialize(location, resources);

        portsButton.setSelected(true);

        portsTable.setPlaceholder(new Label("No ports available. Click on '+' to add new."));

        portsNameCol.setCellValueFactory(new PropertyValueFactory<>("name"));
        numberCol.setCellValueFactory(new PropertyValueFactory<>("number"));
        isPublicCol.setCellValueFactory(new PropertyValueFactory<>("isPublic"));

        deletePortButton.visibleProperty().bind(portsTable.getSelectionModel().selectedItemProperty().isNotNull());
        separator.visibleProperty().bind(portsTable.getSelectionModel().selectedItemProperty().isNotNull());
    }

    public void addPort() throws IOException {
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/addModulesPort.fxml"));
        App.getScene().setRoot(loader.load());
        loader.<AddModulesPortController>getController().loadData(module);
    }

    public void deletePort() {
        ModulePort modulePortToDelete = portsTable.getSelectionModel().getSelectedItem();
        modulePortService.deleteModulePort(modulePortToDelete);
        portsTable.getItems().remove(modulePortToDelete);
    }

    public void changeToEnglishLanguage() throws IOException {
        InternalizationUtility.setLocale(Locale.ENGLISH);
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesPorts.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);    }

    public void changeToCzechLanguage() throws IOException {
        InternalizationUtility.setLocale(new Locale("cs"));
        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesPorts.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);
    }

    public void loadData(Module module) {
        super.loadData(module);
        portsTable.getItems().addAll(modulePortService.getModulePortsByModuleName(module.getName()));
    }
}
