/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.operations;

import com.brunomnsilva.smartgraph.graph.Edge;
import com.brunomnsilva.smartgraph.graph.Graph;
import com.brunomnsilva.smartgraph.graph.GraphEdgeList;
import com.brunomnsilva.smartgraph.graph.Vertex;
import com.brunomnsilva.smartgraph.graphview.*;
import com.jfoenix.controls.JFXCheckBox;
import com.jfoenix.controls.JFXSlider;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.InternalizationUtility;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.PopUpWindow;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.ZoomingPane;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.BaseController;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Node;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.NodeParameter;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Operation;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.ModuleService;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.OperationService;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.StackPane;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.modules.ViewModulesDetailsController;

import java.io.IOException;
import java.util.HashSet;
import java.util.Optional;
import java.util.ResourceBundle;
import java.util.Set;

public class AddOperationController extends BaseController {

    private final OperationService operationService = new OperationService();

    private final ModuleService moduleService = new ModuleService();

    private static SmartGraphPanel<Node, String> graphView;

    private static Graph<Node, String> graph;

    private SmartGraphVertex<Node> selectedVertex;

    private SmartGraphEdge<String, Node> selectedEdge;

    private boolean addingEdgeMode = false;

    private int edgeNumber = 0;

    private Operation operation;

    @FXML
    private JFXCheckBox automaticCheckBox;

    @FXML
    private Button addingEdgeButton;
    @FXML
    private Button deleteElementButton;

    @FXML
    private Label operationNameLabel;
    @FXML
    private Label nameLabel;
    @FXML
    private Label typeLabel;
    @FXML
    private Label healthPortLabel;
    @FXML
    private Label tagLabel;

    @FXML
    private TableView<NodeParameter> nodeParametersTable;
    @FXML
    private TableColumn<NodeParameter,String> nameCol;
    @FXML
    private TableColumn<NodeParameter,String> valueCol;
    @FXML
    private TableColumn<NodeParameter,Boolean> isConstantCol;
    @FXML
    private JFXSlider slider;

    @FXML
    private StackPane graphPane;
    @FXML
    private AnchorPane sidePanel;

    @FXML
    private Button deleteParameterButton;

    @FXML
    private void initialize(){
        showSideBar(false);
        deleteParameterButton.visibleProperty().bind(nodeParametersTable.getSelectionModel().selectedItemProperty().isNotNull());
        nodeParametersTable.setPlaceholder(new Label("Selected node does not have any parameters."));

        addingEdgeButton.setVisible(false);
        deleteElementButton.setVisible(false);

        nameCol.setCellValueFactory(new PropertyValueFactory<>("name"));
        valueCol.setCellValueFactory(new PropertyValueFactory<>("defaultValue"));
        isConstantCol.setCellValueFactory(new PropertyValueFactory<>("isConstant"));

        SmartPlacementStrategy strategy = new SmartCircularSortedPlacementStrategy();
        graph = new GraphEdgeList<>();
        graphView = new SmartGraphPanel<>(graph, strategy);
        new Scene(graphView, 550, 600);
        graphView.init();

        ZoomingPane zoomingPane = new ZoomingPane(graphView);
        zoomingPane.zoomFactorProperty().bind(slider.valueProperty());

        graphPane.getChildren().add(graphView);

        graphView.setVertexDoubleClickAction(this::setSelectedVertex);
        graphView.setEdgeDoubleClickAction(this::setSelectedEdge);

        automaticCheckBox.selectedProperty().bindBidirectional(graphView.automaticLayoutProperty());
        automaticCheckBox.selectedProperty().setValue(true);
    }

    public void setSelectedEdge(SmartGraphEdge<String, Node> edge) {
        showSideBar(false);
        addingEdgeButton.setVisible(false);
        deleteElementButton.setVisible(true);

        deselect();

        edge.addStyleClass("selectedEdge");
        selectedEdge = edge;
    }

    public void setSelectedVertex(SmartGraphVertex<Node> vertex) {
        if(selectedVertex != null && addingEdgeMode){
            if(graph.areAdjacent(selectedVertex.getUnderlyingVertex(), vertex.getUnderlyingVertex())){
                try {
                    PopUpWindow.showAdjacentVertices();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } else {
                Edge<String, Node> e = graph.insertEdge(selectedVertex.getUnderlyingVertex().element(), vertex.getUnderlyingVertex().element(), String.valueOf(edgeNumber++));
                if (hasCycle()){
                    graph.removeEdge(e);
                    try {
                        PopUpWindow.showCycleDetected();
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                } else{
                    graphView.update();
                }
            }
            changeAddingEdgeMode();
        }

        showSideBar(true);
        deleteElementButton.setVisible(true);
        if (graph.numVertices() > 1){
            addingEdgeButton.setVisible(true);
        }

        nodeParametersTable.getItems().clear();
        nodeParametersTable.getItems().addAll(vertex.getUnderlyingVertex().element().getNodeParameters());

        Node node = vertex.getUnderlyingVertex().element();
        nameLabel.setText(node.getModuleName());


        deselect();

        vertex.addStyleClass("selectedVertex");
        selectedVertex = vertex;
    }

    public void deselect() {
        if (selectedEdge != null){
            selectedEdge.removeStyleClass("selectedEdge");
            selectedEdge = null;
        }

        if (selectedVertex != null){
            selectedVertex.removeStyleClass("selectedVertex");
            selectedVertex = null;
        }
    }

    public void showSideBar(boolean state){
        sidePanel.setManaged(state);
        sidePanel.setVisible(state);
    }

    public void closeSideBar(){
        showSideBar(false);
    }

    public void deleteParameter(){
        NodeParameter nodeParameter = nodeParametersTable.getSelectionModel().getSelectedItem();
        selectedVertex.getUnderlyingVertex().element().getNodeParameters().remove(nodeParameter);

        nodeParametersTable.getItems().clear();
        nodeParametersTable.getItems().addAll(selectedVertex.getUnderlyingVertex().element().getNodeParameters());
    }

    public boolean hasCycle(){
        /**
         for (Vertex<Node> node : graph.vertices()) {
            System.out.println(node);
            System.out.println(graph.incidentEdges(node));
        } **/

        Set<Node> visited = new HashSet<>();
        Set<Node> currentlyVisiting = new HashSet<>();

        for (Vertex<Node> node : graph.vertices()) {
            if (!visited.contains(node.element())){
                if (dfs(visited, currentlyVisiting, node)){
                    return true;
                }
            }
        }
        return false;
    }

    public boolean dfs(Set<Node> visited, Set<Node> currentlyVisiting, Vertex<Node> node){
        visited.add(node.element());
        currentlyVisiting.add(node.element());

        for (Edge<String, Node> n : graph.edges()){
            if (n.vertices()[0].equals(node)){
                if (currentlyVisiting.contains(n.vertices()[1].element()) ||
                        dfs(visited, currentlyVisiting, n.vertices()[1])){
                    return true;
                }
            }
        }
        currentlyVisiting.remove(node.element());
        return false;
    }

    public void addNodeParameter() throws IOException {
        Node node = selectedVertex.getUnderlyingVertex().element();
        Optional<NodeParameter> nodeParameter = PopUpWindow.addNodeParameter(node);

        if (nodeParameter.isEmpty()) {
            return;
        }
        node.getNodeParameters().add(nodeParameter.get());
        nodeParametersTable.getItems().add(nodeParameter.get());
    }

    public void showModuleDetails() throws IOException {
        Module module = moduleService.getModuleByName(selectedVertex.getUnderlyingVertex().element().getModuleName()).get(0);

        ResourceBundle resources = ResourceBundle.getBundle("internalizationBundles/languageBundle", InternalizationUtility.getLocale());
        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/viewModulesPorts.fxml"), resources);
        App.getScene().setRoot(loader.load());
        loader.<ViewModulesDetailsController>getController().loadData(module);
    }

    public void deleteElement(){
        showSideBar(false);

        if (selectedVertex != null) {
            for (Edge<String, Node> e : graph.incidentEdges(selectedVertex.getUnderlyingVertex())){
                graph.removeEdge(e);
            }
            graph.removeVertex(selectedVertex.getUnderlyingVertex());
            operation.removeNode(selectedVertex.getUnderlyingVertex().element());
            selectedVertex = null;
        }

        if (selectedEdge != null) {
            graph.removeEdge(selectedEdge.getUnderlyingEdge());
            selectedEdge = null;
        }

        deleteElementButton.setVisible(false);
        addingEdgeButton.setVisible(false);

        graphView.update();
    }

    public void save() throws IOException {

        operation.setName(operationNameLabel.getText());
        operation.getEdges().clear();

        Set<cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Edge> edges = new HashSet<>();
        operationService.addOperation(operation);

        for (Edge<String, Node> e : graph.edges()){
            Node fromNode = operation.getNodes().stream().filter(node -> node.getModuleName().equals(e.vertices()[0].element().getModuleName())).findFirst().get();
            Node toNode = operation.getNodes().stream().filter(node -> node.getModuleName().equals(e.vertices()[1].element().getModuleName())).findFirst().get();

            System.out.println(fromNode);
            System.out.println(toNode);

            cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Edge edge = new cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Edge(operation, fromNode.getID(), toNode.getID());
            edges.add(edge);
        }
        operation.setEdges(edges);
        operationService.addOperation(operation);

        viewOperations();
    }

    public void changeAddingEdgeMode(){
        addingEdgeMode = !addingEdgeMode;
        if (addingEdgeMode){
            addingEdgeButton.getStyleClass().add("defaultButtonSelected");
        } else{
            addingEdgeButton.getStyleClass().remove("defaultButtonSelected");
        }
    }

    public void addNode() throws IOException {
        Module selectedModule = PopUpWindow.getModule();

        if (selectedModule != null){
            Node newNode = new Node(operation, selectedModule.getName());
            operation.getNodes().add(newNode);
            graph.insertVertex(newNode);
            graphView.update();
        }
    }

    public void showModuleParameters() throws IOException {
        PopUpWindow.showModuleParameters(selectedVertex.getUnderlyingVertex().element().getModuleName());
    }

    public void changeName() {
        /**
        Optional<String> operationName = PopUpWindow.getOperationName();
        if (operationName.isEmpty()){
            return;
        }
        if (operationService.getOperationByName(operationName.get()).isEmpty()){
            operationNameLabel.setText(operationName.get());
        } else {
            PopUpWindow.showDuplicateNameWindow();
        }
         **/
    }

    public void loadData(Operation operation){
        this.operation = operation;
        operationNameLabel.setText(operation.getName());

        for (Node n : operation.getNodes()){
            graph.insertVertex(n);
        }

        for (cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Edge edge : operation.getEdges()){
            Node from = null;
            Node to = null;
            for (Node node : operation.getNodes()){
                if(edge.getFromNode() == node.getID()){
                    from = node;
                }
                if(edge.getToNode() == node.getID()){
                    to = node;
                }
            }
            graph.insertEdge(from, to, String.valueOf(edgeNumber++));
        }
        graphView.update();
    }

    public void loadData(String operationName){
        this.operation = new Operation();
        operationNameLabel.setText(operationName);
    }
}
