/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.operations;

import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.PopUpWindow;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.controller.BaseController;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Operation;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.service.OperationService;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.Button;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableRow;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.App;

import java.io.IOException;
import java.util.Optional;

public class ViewOperationsController extends BaseController {

    private OperationService operationService = new OperationService();

    @FXML
    private TableView<Operation> table;

    @FXML
    private Button deleteButton;

    @FXML
    private Button showDetailsButton;

    @FXML
    private TableColumn<Module,String> nameCol;

    @FXML
    private void initialize() {
        deleteButton.visibleProperty().bind(table.getSelectionModel().selectedItemProperty().isNotNull());
        showDetailsButton.visibleProperty().bind(table.getSelectionModel().selectedItemProperty().isNotNull());

        nameCol.setCellValueFactory(new PropertyValueFactory<>("name"));

        table.setRowFactory( tv -> {
            TableRow<Operation> row = new TableRow<>();
            row.setOnMouseClicked(event -> {
                if (event.getClickCount() == 2 && (! row.isEmpty()) ) {
                    try {
                        showDetails();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            });
            return row ;
        });

        table.getItems().addAll(operationService.getOperations());
    }

    public void addOperation() throws IOException {

         Optional<String> operationName = PopUpWindow.getOperationName();

        if (operationName.isEmpty()){
            return;
        }

        if (operationService.getOperationByName(operationName.get()).isEmpty()){
            FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/addOperation.fxml"));
            App.getScene().setRoot(loader.load());
            loader.<AddOperationController>getController().loadData(operationName.get());
        } else {
            PopUpWindow.showDuplicateNameWindow();
        }
    }

    public void showDetails() throws IOException{
        Operation operation = table.getSelectionModel().getSelectedItem();

        FXMLLoader loader = new FXMLLoader(getClass().getClassLoader().getResource("view/addOperation.fxml"));
        App.getScene().setRoot(loader.load());
        loader.<AddOperationController>getController().loadData(operation);
    }

    public void delete() throws IOException {
        if(PopUpWindow.getOperationDeleteResult()){
            Operation operationToDelete = table.getSelectionModel().getSelectedItem();
            operationService.deleteOperation(operationToDelete);
            table.getItems().remove(operationToDelete);
        }
    }
}
