/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.databaseAccess;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import org.hibernate.Session;
import org.hibernate.query.Query;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.HibernateConnectionUtility;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Module;

public class ModuleDAO {

    public ObservableList<Module> getAllModule(){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        Query query = session.createQuery("from Module");
        @SuppressWarnings("unchecked")
        ObservableList<Module>  list = FXCollections.observableArrayList(query.list());
        session.getTransaction().commit();
        session.close();
        return list;
    }

    public ObservableList<Module> getModulesSuccessors(String moduleName){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        Query query = session.createQuery("select m.successor from Module m where m.name = :name");
        query.setParameter("name", moduleName);
        @SuppressWarnings("unchecked")
        ObservableList<Module>  list = FXCollections.observableArrayList(query.list());
        session.getTransaction().commit();
        session.close();
        return list;
    }

    public ObservableList<Module> getModuleByName(String moduleName){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        Query query = session.createQuery("from Module m where m.name = :name");
        query.setParameter("name", moduleName);
        @SuppressWarnings("unchecked")
        ObservableList<Module>  list = FXCollections.observableArrayList(query.list());
        session.getTransaction().commit();
        session.close();
        return list;
    }

    public void addModule(Module module){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        session.save(module);
        session.getTransaction().commit();
        session.close();
    }

    public void updateModule(Module module){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        session.update(module);
        session.getTransaction().commit();
        session.close();
    }

    public void addModulesSuccessor(Module parent, Module successor){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        Module parentToAdd =  session.load(Module.class, parent.getName());
        Module successorToAdd =  session.load(Module.class, successor.getName());
        //parentToAdd.getSuccessor().add(successorToAdd);
        //successorToAdd.getParent().add(parentToAdd);
        session.update(parentToAdd);
        session.update(successorToAdd);
        session.getTransaction().commit();
        session.close();
    }

    public void deleteModule(Module module) {
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        session.delete(module);
        session.getTransaction().commit();
        session.close();
    }

}
