/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.databaseAccess;

import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.HibernateConnectionUtility;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import org.hibernate.Session;
import org.hibernate.query.Query;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.ModuleParameter;

public class ModuleParameterDAO {

    public ObservableList<ModuleParameter> getAllModuleParameters(){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        Query query = session.createQuery("from ModuleParameter");
        @SuppressWarnings("unchecked")
        ObservableList<ModuleParameter>  list = FXCollections.observableArrayList(query.list());
        session.getTransaction().commit();
        session.close();
        return list;
    }

    public ObservableList<ModuleParameter> getModuleParametersByModuleName(String moduleName){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        Query query = session.createQuery("SELECT m FROM ModuleParameter m where m.module.name = :name");
        query.setParameter("name", moduleName);
        @SuppressWarnings("unchecked")
        ObservableList<ModuleParameter>  list = FXCollections.observableArrayList(query.list());
        session.getTransaction().commit();
        session.close();
        return list;
    }

    public ObservableList<ModuleParameter> getModuleParametersByName(String name){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        Query query = session.createQuery("SELECT m FROM ModuleParameter m where m.name = :name");
        query.setParameter("name", name);
        @SuppressWarnings("unchecked")
        ObservableList<ModuleParameter>  list = FXCollections.observableArrayList(query.list());
        session.getTransaction().commit();
        session.close();
        return list;
    }

    public void addModuleParameter(ModuleParameter moduleParameter){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        session.save(moduleParameter);
        session.getTransaction().commit();
        session.close();
    }

    public void updateModuleParameter(ModuleParameter moduleParameter){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        session.update(moduleParameter);
        session.getTransaction().commit();
        session.close();
    }

    public void deleteModuleParameter(ModuleParameter moduleParameter) {
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        session.delete(moduleParameter);
        session.getTransaction().commit();
        session.close();
    }
}
