/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.databaseAccess;

import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.HibernateConnectionUtility;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import org.hibernate.Session;
import org.hibernate.query.Query;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Node;
import cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model.Operation;

public class NodeDAO {

    public ObservableList<Node> getNodes(){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        Query query = session.createQuery("from Node");
        @SuppressWarnings("unchecked")
        ObservableList<Node>  list = FXCollections.observableArrayList(query.list());
        session.getTransaction().commit();
        session.close();
        return list;
    }

    public ObservableList<Node> getNodesByOperation(String operationName){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        Query query = session.createQuery("SELECT n FROM Node n where n.operation.name = :name");
        query.setParameter("name", operationName);
        @SuppressWarnings("unchecked")
        ObservableList<Node>  list = FXCollections.observableArrayList(query.list());
        session.getTransaction().commit();
        session.close();
        return list;
    }

    public ObservableList<Operation> getNodeByModuleName(String moduleName){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        Query query = session.createQuery("select n.operation from Node n where n.moduleName = :name");
        query.setParameter("name", moduleName);
        @SuppressWarnings("unchecked")
        ObservableList<Operation> list = FXCollections.observableArrayList(query.list());
        //ObservableSet<Operation> set = FXCollections.emptyObservableSet();
        //CollectionUtils.addAll(set, list);
        session.getTransaction().commit();
        session.close();
        return list;
    }

    public void addNode(Node node){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        session.save(node);
        session.getTransaction().commit();
        session.close();
    }

    public void updateNode(Node node){
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        session.update(node);
        session.getTransaction().commit();
        session.close();
    }


    public void deleteNode(Node node) {
        Session session = HibernateConnectionUtility.getSessionFactory().openSession();
        session.beginTransaction();
        session.delete(node);
        session.getTransaction().commit();
        session.close();
    }
}
