/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model;

import javax.persistence.*;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Entity
@Table(name = "definition_modules")
public class Module implements Serializable, Comparable<Module> {

    @Id
    @Column(name = "name")
    private String name;

    @Column(name = "type")
    private String type;

    @Column(name = "description")
    private String description;

    @Column(name = "is_blocking")
    private boolean isBlocking;

    @Column(name = "health_port")
    private int healthPort;

    @Column(name = "image")
    private String image;

    @Column(name = "tag")
    private String tag;

    @OneToMany(mappedBy = "module", cascade = CascadeType.REMOVE)
    private List<ModuleParameter> moduleParameters = new ArrayList<>();

    @OneToMany(mappedBy = "module", cascade = CascadeType.REMOVE)
    private List<ModulePort> modulePorts = new ArrayList<>();

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public boolean getIsBlocking() {
        return isBlocking;
    }

    public void setIsBlocking(boolean blocking) {
        this.isBlocking = blocking;
    }

    public int getHealthPort() {
        return healthPort;
    }

    public void setHealthPort(int healthPort) {
        this.healthPort = healthPort;
    }

    public String getImage() {
        return image;
    }

    public void setImage(String image) {
        this.image = image;
    }

    public String getTag() {
        return tag;
    }

    public void setTag(String tag) {
        this.tag = tag;
    }

    public List<ModuleParameter> getModuleParameters() {
        return moduleParameters;
    }

    public void setModuleParameters(List<ModuleParameter> moduleParameters) {
        this.moduleParameters = moduleParameters;
    }

    public List<ModulePort> getModulePorts() {
        return modulePorts;
    }

    public void setModulePorts(List<ModulePort> modulePorts) {
        this.modulePorts = modulePorts;
    }

    @Override
    public String toString() {
        return getName();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Module module = (Module) o;
        return name.equals(module.name);

    }

    @Override
    public int hashCode() {
        return Objects.hash(name);
    }


    @Override
    public int compareTo(Module module) {
        if (this.equals(module)){
            return 0;
        }
        return -1;
    }
}
