/*
 Copyright 2020 Masarykova univerzita

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0


Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

package cz.fi.muni.ANALYZA.db_orchestrator_filler.GUI.model;

import javax.persistence.*;
import java.io.Serializable;
import java.util.*;

@Entity
@Table(name = "definition_nodes")
public class Node implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE)
    @Column(name = "id")
    private int ID;

    @ManyToOne
    @JoinColumn(name = "operation_name", referencedColumnName = "name")
    private Operation operation;

    @Column(name = "description")
    private String description;

    @Column(name = "module_name")
    private String moduleName;

    @OneToMany(mappedBy = "node", cascade = CascadeType.ALL, fetch = FetchType.EAGER, orphanRemoval = true)
    private Set<NodeParameter> nodeParameters = new HashSet<>();

    public Node() {
    }

    public Node(Operation operation, String moduleName) {
        this.operation = operation;
        this.moduleName = moduleName;
    }

    public int getID() {
        return ID;
    }

    public void setID(int ID) {
        this.ID = ID;
    }

    public String getModuleName() {
        return moduleName;
    }

    public void setModuleName(String moduleName) {
        this.moduleName = moduleName;
    }

    public Operation getOperation() {
        return operation;
    }

    public void setOperation(Operation operation) {
        this.operation = operation;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Set<NodeParameter> getNodeParameters() {
        return nodeParameters;
    }

    public void setNodeParameters(Set<NodeParameter> nodeParameters) {
        this.nodeParameters = nodeParameters;
    }

    public void removeNodeParameter(NodeParameter nodeParameter){
        nodeParameters.remove(nodeParameter);
        nodeParameter.setNode(null);
    }

    @Override
    public String toString() {
        return moduleName;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Node node = (Node) o;
        return ID == node.ID &&
                Objects.equals(operation, node.operation) &&
                Objects.equals(moduleName, node.moduleName);
    }

    @Override
    public int hashCode() {
        return Objects.hash(ID, operation, moduleName);
    }
}
